TEfficiency *Efficiency(const char *fname, int serialrowmin, int serialrowmax)
{
	/// Cuts ////
	const double kZDistCut    =   5.; // [mm]
	const double kRphiDistCut = 200.; // [mm]
	const double kRphiDistMax =  10.; // [mm] : hist limit
	const int    kNbin        =  100;

	/// Get run number ///
	string sfname(fname);
	auto run = -9999;
	regex target(R"(.*/*r([0-9]+).*\.root)"); // "pattern: [.../]r(run)....root"
	smatch match; // match results (0:the whole matched string, 1~:submatch)
	if (regex_search(sfname, match, target)) {
		run = atoi(match[1].str().data());
		cerr << "run = " << run << endl;
	} else {
		cerr << "file name does not contain run #, quit." << endl;
		abort();
	}

	/// Open input file ///
	auto filep  = new TFile(sfname.data());
	auto hits2p = static_cast<TNtuple *>(filep->Get("MyRootFileProcessor2/hits2"));
	
	/// Tuple item to draw ///
	stringstream ssitem;
	ssitem << "abs(rphixp1-rphixp2)"; // rphi distance of 2 tracks

	/// Output histograms for total and accepted events
	stringstream sshall;
	sshall << "hALL" << serialrowmin << "_" << serialrowmax << "_r" << run;
	stringstream sshok;
	sshok  << "hOK"  << serialrowmin << "_" << serialrowmax << "_r" << run;

	/// Set targets to draw
	stringstream sstarget1;
	sstarget1 << ssitem.str() << ">>" << sshall.str() 
			          << "("  << kNbin <<  ",0.," << kRphiDistMax << ")" 
				  << ends;
	stringstream sstarget2;
	sstarget2 << ssitem.str() << ">>" << sshok.str() 
			          << "("  << kNbin <<  ",0.," << kRphiDistMax << ")" 
				  << ends;

	cerr << "target1:" << sstarget1.str() << endl
	     << "target2:" << sstarget2.str() << endl;

	/// Cuts ////
	stringstream sscutall;
	sscutall << ssitem.str() << " < " << kRphiDistCut 
			         << " && serialrow >= " << serialrowmin
			         << " && serialrow <= " << serialrowmax
			         << " && trkid <= 0"
			         << " && abs(zxp1-zxp2) <" << kZDistCut
			         << ends;

	stringstream sscutok;
	sscutok  << ssitem.str() << " < " << kRphiDistCut 
			         << " && serialrow >= " << serialrowmin
			         << " && serialrow <= " << serialrowmax
			         << " && (trkid == 0 || trkid == 1)"
			         << " && abs(zxp1-zxp2) <" << kZDistCut
			         << " && ndf > 0"
			         << " && nktrks == 2"
			         << " && ((ismulti == 1 && issplit == 1) || ismulti == 0) "
			         << " && nhits > 0"
			         << ends;

	/// Draw targets ///
	hits2p->Draw(sstarget1.str().data(), sscutall.str().data());
	hits2p->Draw(sstarget2.str().data(), sscutok.str().data());

	/// Calculate efficiency ///
	auto &hALL  = *static_cast<TH1 *>(gROOT->FindObject(sshall.str().data()));
	auto &hOK   = *static_cast<TH1 *>(gROOT->FindObject(sshok.str().data()));
#if 0
#else
	hALL.Scale(2.);
#endif
	auto  hEffp = new TEfficiency(hOK, hALL);

	return hEffp;
}

TEfficiency *Efficiency(int run = 19978, int serialrowmin = 44, int serialrowmax = 44, const char *dir = "../Data/Refined")
{
	/// Guess input file path from run # ///
	stringstream ssfname;
	ssfname << dir << "/r" << run << "_fakeTwoTrack.root" << ends;

	auto hEffp = Efficiency(ssfname.str().data(), serialrowmin, serialrowmax);
	stringstream ssefftitle;
	ssefftitle << "2-hit separation efficiency for serial row #" << serialrowmin << "-" << serialrowmax
		   << "; 2-hit distance [mm]; Efficieny" << ends;
	hEffp->SetTitle(ssefftitle.str().data());

	return hEffp;
}
