class MultiGaus {
	public:
		MultiGaus(int npar = 6) : _npar(npar) {};
		double operator()(double *x, double *par);

	private:
		int _npar;
};

TH1D* Fit(std::vector<double> chargeVec,
          std::vector<double> dchargeVec,
	  double              chg1,
	  double              mean1,
	  double              sigma1,
	  double              chg2,
	  double              mean2,
	  double              sigma2,
	  double              rphimin,
	  double              rphimax,
	  double              wpad)
{
	auto npads = chargeVec.size();
	auto histp = new TH1D("hchgdist","",npads,rphimin,rphimax);
	auto &hist = *histp;
	for (auto ibin = 0; ibin < npads; ibin++) {
		hist.SetBinContent(ibin+1, chargeVec[ibin]);
#if 0
		hist.SetBinError  (ibin+1, 3.);
#else
		hist.SetBinError  (ibin+1, dchargeVec[ibin]);
#endif
		cerr << ibin+1 << " " << chargeVec[ibin] << endl;
	}
	hist.SetMinimum(0.);

	auto npar = 3*2;

	MultiGaus multig(npar);

	auto funp = new TF1("multigaus", multig, rphimin, rphimax, npar);
	auto &fun = *funp;
#if 1
	fun.SetParameter(0    , chg1);
	fun.SetParameter(1    , mean1);
	fun.FixParameter(2    , sigma1);
#else
	fun.FixParameter(0    , chg1);
	fun.FixParameter(1    , mean1);
	fun.FixParameter(2    , sigma1);
#endif
	fun.SetParLimits(0,      0.,    1.e6);
	fun.SetParLimits(1, rphimin, rphimax);
#if 1
	fun.SetParameter(3    , chg2);
	fun.SetParameter(4    , mean2);
	fun.FixParameter(5    , sigma2);
#else
	fun.FixParameter(3    , chg2);
	fun.FixParameter(4    , mean2);
	fun.FixParameter(5    , sigma2);
#endif

	fun.SetParLimits(3,      0.,    1.e6);
	fun.SetParLimits(4, rphimin, rphimax);

	//hist.Add(funp);

	hist.Fit(&fun, "I", "goff", rphimin, rphimax);

	return histp;
}

/// Multi-gaussian fitting function
double MultiGaus::operator()(double *xp, double *par)
{
	auto sum = 0.;
	for (int itrk = 0; itrk < _npar/3; itrk++) {
		auto peak  = par[3*itrk  ];
		auto mean  = par[3*itrk+1];
		auto sigma = par[3*itrk+2];
		auto x = xp[0];
		sum += peak*exp(-0.5*pow((x-mean)/sigma, 2));
	}
	return sum;
}
