//LCIO
#include "lcio.h"
#include "IO/LCWriter.h"
#include "EVENT/LCIO.h"
#include "IMPL/LCEventImpl.h"
#include "IMPL/LCGenericObjectImpl.h"
#include "IMPL/LCCollectionVec.h"
#include "IMPL/LCTOOLS.h"
#include "UTIL/LCTime.h"
//LCCD
#include "lccd.h"
#include "lccd/DBInterface.hh"
//GEAR
#include "gear/TPCParameters.h"
#include "gear/TPCModule.h"
#include "gearxml/GearXML.h"
#include "gear/GearMgr.h"
#include "gear/GEAR.h"
//TPCCONDDATA
#include "ADCChannelMapping.h"

//C++
#include <iostream>
#include <string>

void printUsage()
{
    std::cout<<"Usage:"<<std::endl;
    std::cout<<"To write data to a lcio simple file:"<<std::endl;
    std::cout<<"  DummyMappingGenerator gearfile.xml [outfile.slcio]" <<std::endl;
    std::cout<<"  (outfile name is optional, default is TPCChannelMapping.slcio)"<< std::endl;
}



int main(int argc, char **argv)
{
    std::string outFileName;
    lcio::LCWriter* lcWrt=nullptr;
    lcio::LCEventImpl*  evt=nullptr;
    if (argc != 2 && argc != 3 )
    {
        printUsage();
        return(1);
    }
    
    //DEBUG
    for (int i=0; i< argc ; ++i)
    {
        std::cout<<"DEBUG: argument ["<<i<<"]: "<<argv[i]<<std::endl;
    } 
    
    // read pad layout from gear
    gear::GearXML gearXML( argv[1] ) ;
    gear::GearMgr* gearMgr = gearXML.createGearMgr() ;
    gear::TPCParameters const &tpcParams=gearMgr->getTPCParameters();

    // get outfile name
    if (argc==3)
        outFileName=argv[2];
    else
        outFileName="TPCChannelMapping.slcio";
    // open outfile
    lcWrt = lcio::LCFactory::getInstance()->createLCWriter() ;
    lcWrt->open( outFileName, lcio::LCIO::WRITE_NEW ) ;
    // create new lcio event
    evt = new lcio::LCEventImpl() ;

    // create the collection we want to store the data in
    lcio::LCCollectionVec* mappingCol = new lcio::LCCollectionVec( lcio::LCIO::LCGENERICOBJECT );
    
    auto modules = tpcParams.getModules();
    // loop over modules
    for (auto module : modules) {
	auto moduleID = module->getModuleID();
        auto nrows = module->getNRows();
	// loop over rows
        for (int row=0; row<nrows; row++) {
            auto padsInRow = module->getPadsInRow(row); 
	    // loop over pads
	    for (auto pad_index : padsInRow) {
                auto pad = module->getPadNumber(pad_index);
                tpcconddata::ADCChannelMapping * channelmapping
#if 0
                    = new tpcconddata::ADCChannelMapping(hw_channel,  pad_index , -1, moduleID, hw_group);
#else
                    = new tpcconddata::ADCChannelMapping(pad_index,  pad_index , -1, moduleID, moduleID);
#endif
                std::cout << "(module,row,pad,id0)=(" 
	 	          << moduleID << ", "
	 	          << row << ", "
	 	          << pad << ", "
			  << pad_index <<")" << std::endl;
                mappingCol->addElement( channelmapping );
	    }
	}
    }

    // add the mapping collection to the event
    evt->addCollection( mappingCol , "TPCChannelMapping" ) ;
    // write the event to the file
    lcWrt->writeEvent( evt ) ;
    // dump the event to the screen
    lcio::LCTOOLS::dumpEvent( evt ) ;
    lcWrt->close() ;
    // ------------ IMPORTANT ------------- !
    // we created the event so we need to delete it ...
    delete evt ;
    // -------------------------------------

    // That's it
    return 0;
}
