#define CHECK_DIST
#include <iomanip>
#include "Runinfo.h"
#ifndef __CINT__
#include <sstream>
#include "TROOT.h"
#include "TSystem.h"
#include "TFile.h"
#include "TNtupleD.h"
#include "TH2.h"
#include "TTree.h"

#include "TPaveText.h"
#include "TLatex.h"
#include "TGraphErrors.h"
#include "TCanvas.h"
#include "TF1.h"
#include "TStyle.h"
#endif

void Process(Int_t run, Int_t layermin, Int_t layermax, Double_t delrphimin, Double_t delrphimax,
             Double_t &x, Double_t &y, Double_t &dx, Double_t &dy, const Char_t *inout);

void GMResol3(Int_t run = 19978, Int_t module = 3, Int_t rowmin = 15, Int_t rowmax = 18)
{
//--Set Style
  // Axis
  gStyle->SetLabelFont(22,"X");
  gStyle->SetLabelFont(22,"Y");
  // Title offset
  gStyle->SetTitleOffset(1.0, "X");
  gStyle->SetTitleOffset(1.2, "Y");
  // Pad grid
  gStyle->SetPadGridX(kTRUE);
  gStyle->SetPadGridY(kTRUE);
  // Title
  gStyle->SetTitleFont(22, "X");
  gStyle->SetTitleFont(22, "Y");
  gStyle->SetTitleFont(22, "");

  const Int_t kNp = 12;
  Double_t xdata[kNp], ydata[kNp], dxdata[kNp], dydata[kNp];
  Double_t kDelRphi[][2] = {{1.0, 2.0}, {2.0, 2.5}, {2.5, 3.0}, {3.0, 3.2}, {3.2, 3.4},
                         {3.4, 3.6}, {3.6, 3.8}, {3.8, 4.0}, {4.0, 4.5}, {4.5, 5.0},
                         {5.0, 6.0}, {6.0, 10.}};

  Int_t layermin = (module < 2 ? 0 : module < 5 ? 28 : 56) + rowmin;
  Int_t layermax = (module < 2 ? 0 : module < 5 ? 28 : 56) + rowmax;

  // ------------------------
  // Loop over 2-hit distance
  // ------------------------
  Int_t n = 0;
  for (Int_t i=0; i<kNp; i++) {
    Double_t delrphimin = kDelRphi[i][0];
    Double_t delrphimax = kDelRphi[i][1];
    Double_t xin, yin, dxin, dyin;
    Process(run, layermin, layermax, delrphimin, delrphimax, xin, yin, dxin, dyin, "in");

    Double_t xot, yot, dxot, dyot;
    Process(run, layermin, layermax, delrphimin, delrphimax, xot, yot, dxot, dyot, "ot");

    Double_t y  = TMath::Sqrt(yin*yot);
    Double_t dy = (dyin+dyot)/2;

    auto x  = (delrphimin+delrphimax)/2.;
    auto dx = 0.;
    xdata[n] = x; ydata[n] = y; dxdata[n] = dx; dydata[n] = dy; n++;
    cerr << " sigma = " << y << "+/-" << dy << endl;
  }
  // ------------------------------
  // Fill delrphi v.s. sigma_x plot
  // ------------------------------
  TCanvas *c1 = new TCanvas("c1","",800,500);
  c1->cd();

  Runinfo &rinfo = *Runinfo::GetInstancePtr();
  auto dlen  = rinfo.GetDlength(run) * 10; // [cm] to [mm]
  TGraphErrors *grp = new TGraphErrors(n,xdata,ydata,dxdata,dydata);
  stringstream titlestr;
  titlestr << "GM Resolutin (Module" << module << " Row" << rowmin << "-" << rowmax << " dlen: " << dlen << "[mm])" << ends;
  grp->SetTitle(titlestr.str().data());
  grp->GetHistogram()->GetXaxis()->SetLimits(0.,10.);
  grp->GetHistogram()->GetXaxis()->SetTitle("2-hit distance [mm]");
  grp->GetHistogram()->GetYaxis()->SetTitle("#sigma_{x} [mm]");
  grp->Draw("ap");
  grp->SetMarkerColor(1);
  grp->SetMarkerStyle(21);
  double ymin = 0.0;
  //double ymax = 0.8;
  //double ymax = 0.3;
  double ymax = 0.5;
  grp->SetMinimum(ymin);
  grp->SetMaximum(ymax);
#if 0
  TF1 fun("fun","sqrt([0]**2+[1]**2*x)");
  fun.SetLineStyle(2);
  fun.SetLineColor(2);
  //grp->Fit(&fun,"","",10.,600.);
  //grp->Fit(&fun,"","",100.,600.);
  //grp->Fit(&fun,"","",150.,600.);
  grp->Fit(&fun,"","",200.,600.);
  Double_t sigma0        = fun.GetParameter(0) * 1.e3;             // [#mum]
  Double_t dsigma0       = fun.GetParError (0) * 1.e3;             // [#mum]
  Double_t cdbyrootneff  = fun.GetParameter(1) * sqrt(10.) * 1e3;  // [#mum/#sqrt{cm}]
  Double_t dcdbyrootneff = fun.GetParError (1) * sqrt(10.) * 1e3;  // [#mum/#sqrt{cm}]
  Double_t chi2          = fun.GetChisquare();
  Int_t    ndf           = fun.GetNDF();

  TPaveText *pt = new TPaveText(330,3.9*ymax/5, 530.,4.9*ymax/5);

  pt->SetFillColor(0);
  pt->SetTextFont(132);
  pt->SetTextAlign(12);
  stringstream stext;
  stext << "#chi^{2}/ndf = " << setprecision(3) << chi2 << "/" << ndf << ends;
  pt->AddText(stext.str().data());
  stext.str("");
  stext.clear();
  pt->Draw();
  stext << "#sigma_{0} = " << setw(6) << setprecision(3) << sigma0
        << " #pm " << setw(4) << setprecision(2) << dsigma0
        << " [#mum]" << ends;
  pt->AddText(stext.str().data());
  stext.str("");
  stext.clear();
  stext << "C_{D}/#sqrt{N_{eff}} = " << setw(6) << setprecision(3) << cdbyrootneff 
        << " #pm " <<  setw(4) << setprecision(2) << dcdbyrootneff 
        << " [#mum/#sqrt{cm}]" << ends;
  pt->AddText(stext.str().data());
  pt->Draw();

  TLatex *fitfun = new TLatex;
  fitfun->SetTextFont(132);
  fitfun->SetTextAlign(12);
  fitfun->DrawLatex(100,4*ymax/5, "#sigma_{x} = #sqrt{#sigma_{0}^{2}+(C_{D}^{2}/N_{eff}) z}");
#endif

#if 0
  // save plot as pdf  file
  Runinfo &rinfo = *Runinfo::GetInstancePtr();
  stringstream ofile;
  ofile << "GMResol_Row" << layer << "_B" << rinfo.GetBfield(run) << "T"
        << "_P" << rinfo.GetMomentum(run) << "GeV.pdf"<< ends; 
  c1->Print(ofile.str().data());
#endif
}

void Process(Int_t run, Int_t layermin, Int_t layermax, Double_t delrphimin, Double_t delrphimax,
             Double_t &x, Double_t &y, Double_t &dx, Double_t &dy, const Char_t *inout)
{
  // ---------------
  // Reset Run Info.
  // ---------------
  Runinfo &rinfo = *Runinfo::GetInstancePtr();

#ifdef CHECK_DIST
  auto delrphi = (delrphimin+delrphimax)/2.;
  stringstream hstr;
  hstr << "c" << "_" << delrphi << "_" << inout << ends;
  TCanvas *cp = new TCanvas(hstr.str().data(),hstr.str().data(),400,400);
  cp->cd();
#endif

  // ---------------
  // Open input file
  // ---------------
  stringstream finstr;
#if 0
  finstr << "../Data/r" << run << ".root" << ends;
#else
  //finstr << "../Data/r" << run << "_fakeTwoTrack.root" << ends;
  finstr << "../r" << run << "_fakeTwoTrack.root" << ends;
#endif
  TFile *hfp = new TFile(finstr.str().data());

  // -----------------
  // Get residual data
  // -----------------
#if 0
  TNtupleD *hResXin = static_cast<TNtupleD *>(hfp->Get("MyRootFileProcessor2/hits"));
#else
  TNtupleD *hResXin = static_cast<TNtupleD *>(hfp->Get("MyRootFileProcessor2/hits2"));
#endif

  stringstream item;
  item << "dx" << inout << ends;

  stringstream cut;

  ///// Cuts ////////////////////////
#if 1
  //const Double_t kResXmin   = -0.8;
  //const Double_t kResXmax   = +0.8;
  const Double_t kResXmin   = -1.2;
  const Double_t kResXmax   = +1.2;
#if 0
  const Int_t    kNdfCut    = 155;
  const Double_t kChi2Cut   = 4000.;
  const Double_t kCpaMinCut = -0.2;
  const Double_t kCpaMaxCut =  0.8;
#else
#if 0
  //const Int_t    kNdfCut    = 80;
  const Int_t    kNdfCut    = 40;
  const Double_t kChi2Cut   = 40000.;
  const Double_t kCpaMinCut = -8.;
  const Double_t kCpaMaxCut =  2.;
#else
  const Int_t    kNdfCut    = 10;
  const Double_t kChi2Cut   = 400000.;
  const Double_t kCpaMinCut = -9999.;
  const Double_t kCpaMaxCut =  9999.;
#endif
#endif
  //const Double_t kPhi0MinCut = -9999.;
  //const Double_t kPhi0MaxCut = +9999.;
#if 1 // phi=0
#if 0
  const Double_t kPhi0MinCut = -0.02;
  const Double_t kPhi0MaxCut = +0.02;
#else
  const Double_t kPhi0MinCut = -0.9;
  const Double_t kPhi0MaxCut = +0.9;
#endif
  //const Double_t kPhi0MinCut = 4.64;
  //const Double_t kPhi0MaxCut = +4.72;
  //const Double_t kPhi0MaxCut = +4.685;
#else // phi=-20deg
  const Double_t kPhi0MinCut = 4.35;
  const Double_t kPhi0MaxCut = 4.4;
#endif
  const Double_t kTanlMinCut = -9999.;
  const Double_t kTanlMaxCut = +9999.;
#if 0
  const Int_t    kNtrksCut   =      1;
#else
  const Int_t    kNtrksCut   =   9999;
#endif
#else
  const Double_t kResXmin   = -1.0;
  const Double_t kResXmax   = +1.0;
  const Int_t    kNdfCut    = 24;
  //const Double_t kChi2Cut   = 300.;
  const Double_t kChi2Cut   = 100.;
  //const Double_t kCpaMinCut = -0.2;
  //const Double_t kCpaMaxCut =  0.8;
  //const Double_t kCpaMinCut = -4.0;
  //const Double_t kCpaMaxCut =  4.0;
  const Double_t kCpaMinCut = -0.6;
  const Double_t kCpaMaxCut =  1.0;
  //const Double_t kPhi0MinCut = 4.64;
  //const Double_t kPhi0MaxCut = 4.70;
  const Double_t kPhi0MinCut = 4.66;
  const Double_t kPhi0MaxCut = 4.695;
  //const Double_t kPhi0MinCut = 4.655;
  //const Double_t kPhi0MaxCut = 4.685;
  const Double_t kTanlMinCut = -0.008;
  const Double_t kTanlMaxCut =  0.005;
  const Int_t    kNtrksCut   =      1;
#endif
  ///////////////////////////////////
  cut << "serialrow>=" << layermin << "&&"
      << "serialrow<=" << layermax << "&&"
      << "nhits==1" << "&&"
      << item.str().data() << ">" << kResXmin << "&&"
      << item.str().data() << "<" << kResXmax << "&&"
      << "ndf>"  << kNdfCut    << "&&"
      << "chi2<" << kChi2Cut   << "&&"
      //<< "fi0>"  << kPhi0MinCut << "&&"
      //<< "fi0<"  << kPhi0MaxCut << "&&"
      //<< "fi0loc>"  << kPhi0MinCut << "&&"
      //<< "fi0loc<"  << kPhi0MaxCut << "&&"
#if 0
      << "incidentphi>"  << kPhi0MinCut << "&&"
      << "incidentphi<"  << kPhi0MaxCut << "&&"
#else
      << "fiinc>"  << kPhi0MinCut << "&&"
      << "fiinc<"  << kPhi0MaxCut << "&&"
#endif
      << "nktrks == 2 &&"
      << "(ismulti == 0 || issplit > 0) &&"
      << "nhits > 0 &&"
      << "abs(rphixp2-rphixp1)>=" << delrphimin << "&&"
      << "abs(rphixp2-rphixp1)<"  << delrphimax << "&&"
      << "abs(zxp2-zxp1)<5. &&"
      << "tnl>"  << kTanlMinCut << "&&"
      << "tnl<"  << kTanlMaxCut;
  if (rinfo.GetBfield(run)) {
    cut << "&&"
        << "cpa>"  << kCpaMinCut << "&&"
        << "cpa<"  << kCpaMaxCut;
  }
  cut   << "&&" 
        << "ntrks<=" << kNtrksCut;
  cut << ends;
  stringstream target;
  //target << item.str().data() << ">>htemp(100," << kResXmin << ", " << kResXmax << ")" << ends;
  //target << item.str().data() << ">>htemp(50," << kResXmin << ", " << kResXmax << ")" << ends;
  //target << item.str().data() << ">>htemp(25," << kResXmin << ", " << kResXmax << ")" << ends;
  target << item.str().data() << ">>htemp(30," << kResXmin << ", " << kResXmax << ")" << ends;
  hResXin->Draw(target.str().data(), cut.str().data());
  TH1 *htemp = static_cast<TH1 *>(gROOT->FindObject("htemp"));

  // -----------------
  // Fit residuals
  // -----------------
  htemp->Rebin(1);
  //htemp->Fit("gaus","","",-0.3,0.3);
  htemp->Fit("gaus","RI","",-1.3,1.3);
  Double_t mu  = htemp->GetFunction("gaus")->GetParameter(1);
  Double_t sg  = htemp->GetFunction("gaus")->GetParameter(2);
  htemp->Fit("gaus","RI","",mu-2.5*sg,mu+2.5*sg);
  //htemp->Fit("gaus","RI","",mu-2.8*sg,mu+2.8*sg);

  x  = rinfo.GetDlength(run) * 10; // [cm] to [mm]
  y  = htemp->GetFunction("gaus")->GetParameter(2); // sigma
  dx = 0;
  dy = htemp->GetFunction("gaus")->GetParError(2);  // its error
#ifdef CHECK_DIST
  htemp->SetMinimum(-0.1);
  htemp->SetMarkerStyle(4);
  htemp->SetMarkerSize(0.7);
  htemp->SetMarkerColor(2);
  htemp->SetLineColor(2);
  htemp->Draw("pe");
#endif
}
