#!/bin/sh
# ============
#  manyrun.sh
# ============
#
# (Function)
#   Reconstruct multiple data files 
# (Usage)
#   manyrun <maxevt>
# (Upadate Record)
#   2011/12/30   K.Fujii         Original version.
#
#--
#  Check arguments.
#--
if [ $# -ge 2 ]; then
      echo "Usage:"
      echo "  ./manyrun_gen <maxevt>"
      echo " where <maxevt> is the maximum number of events to analyaze"
      echo " per run with its default value being 20000."
      exit 0
#--
# Set maximum number of events to reconstruct per run
#--
elif [ $# -eq 1 ]; then
      echo "maxevt is set to $1"
      maxevt=$1
else
      maxevt=20000
fi
echo "Going to generate ${maxevt} events per run"
#--
# Loop over runs
#--
fun()
{
	declare -a a=($1)
	echo ${a[@]}
}
yoff=4.
ee=5.
phi=0.
lamb=0.
declare -i nrun=15
declare -a runs=($(fun "19987 19985 19984 19983 19982 19981 19980 19979 19978 19977 19976 19975 19974 19973 19972"))
declare -a zees=($(fun "572.64 560.14 535.14 510.14 485.14 460.14 435.14 385.14 335.14 285.14 235.14 185.14 135.14 85.14 35.14"))
echo The size of runs is ${#runs[@]}

declare -i i
for ((i=0;i<${nrun};i++)); do
	run=${runs[i]}
	zee=${zees[i]}
	echo ".............................."
	echo "Start generating run ${run} z ${zee} mm ..."
	fname="electron5GeV.singletrack.r${run}.slcio"

	cmd="generateElectronsForLP1 ${maxevt} ${yoff} ${zee} ${ee} ${phi} ${lamb} ${fname} > gen.r$run.${ee}GeV.z${z}mm.log 2>&1 &"
	eval ${cmd}
done
wait
#--
# That's it, folks!
#--
exit 0
