#ifndef EVENT_MERGER_PROCESSOR_H
#define EVENT_MERGER_PROCESSOR_H

#include "marlin/Processor.h"
#include "lcio.h"
#include <string>
#include <map>

namespace EVENT{
    class LCCollection ;
}

namespace eventmerger {

    class EventMergerProcessor : public marlin::Processor {
        public:
            EventMergerProcessor();
  
            virtual marlin::Processor * newProcessor() { return new EventMergerProcessor; }
        
            virtual const std::string & name() const   { return Processor::name(); }
        
            virtual void init();
        
            virtual void processRunHeader(LCRunHeader * run);
  
            virtual void processEvent(EVENT::LCEvent  * evt); 
        
            virtual void check(EVENT::LCEvent * evt); 
        
            virtual void end();
  
        private:
            /// utility functions
            inline long long cellID2long      (int id0, int id1) { return ((long long)id0 << 32) | id1;       }
            inline int       cellID0fromlongID(long long longID) { return (int)(longID >> 32);                }
            inline int       cellID1fromlongID(long long longID) { return (int)(longID & 0x00000000ffffffff); }
  
            void merge(EVENT::LCCollection * src1p, // 1st source collection
                       EVENT::LCCollection * src2p, // 2nd source collection
                       EVENT::LCCollection * outp); // output merged collection
  
            void fillADC(EVENT::LCCollection                           * col,     // TrackerRawData collection
                         std::map<long long, std::vector<short int> *> & ADCMap); // channel to adc data map
  
        protected:
            std::string      _inputTrackColName; // input collection name
            std::string      _inputColName;      // input collection name
            std::string      _outputColName;     // output collection name
  
            IO::LCReader    *_lcReader;          // pointer to LC reader instance
  
            int              _nRun;              // run number
            int              _nEvt;              // event number
            int              _nMergedEvt;        // merged event number
            int              _nTracksMin;        // minimum # tracks before merging
            int              _nTracksMax;        // maximum # tracks before merging
            int              _nDfMin;            // minimum # degrees of freedom
    };  
} // end of namespace eventmerger
#endif  
