#ifndef ROOT_FILE_PROCESSOR2_H
#define ROOT_FILE_PROCESSOR2_H 1

#include "marlin/Processor.h"
#include "lcio.h"

#include <string>

using namespace lcio ;
using namespace marlin ;

class TObjArray;
class TVector3;
class EXTPCHit;
class EXHYBTrack;

#ifdef MARLIN_USE_AIDA
namespace AIDA {
  class ITuple;
}
#endif

namespace gear {
  class TPCModule;
}

namespace EVENT {
  class LCCollection;
  class TrackerHit;
  class Track;
}

class RootFileProcessor2 : public Processor {
 public:
  /// tuple types to save local event info.
  
  using TrackInfo_t = std::tuple<int   ,  // ndf
        			 double,  // chi2	
  			         double,  // fi0
  			         double,  // cpa
  			         double,  // tnl
  			         int   ,  // ntrks
  			         int   ,  // trkid
  			         int   ,  // evt
  			         int   >; // mevt
  
  using HitInfo_t = std::tuple<double,	// dxin
  		       	       double,  // dxot
  		  	       double,	// dzin
  		       	       double,  // dzot
  		       	       double,  // chg
  		       	       int   ,  // ndf
  		       	       double,  // chi2
  		       	       double,  // fi0
  		       	       double,  // cpa
  		       	       double,  // tnl
  		       	       int   ,  // nhits
  		       	       int   ,  // ntrks
  		       	       double,  // fi0loc
  		       	       double,  // fiinc
  		       	       double,  // rphic
  		       	       double,  // trkid
  		       	       int   ,  // evt
			       double,  // zc
  		       	       int   ,  // mevt
  		       	       double,  // rphixp1
  		       	       double,  // zxp1
  		       	       double,  // rphixp2
  		       	       double,  // zxp2
			       int   ,  // ismulti
			       int   >; // issplit
  //--
  // Enumerator for HitInfoMap_t
  //--
  enum EHitInfoFiels { EHdxin , EHdxot   , EHdzin  , EHdzot   , EHchg    ,
  		       EHndf  , EHchi2   , EHfi0   , EHcpa    , EHtnl    ,
  		       EHnhits, EHntrks  , EHfi0loc, EHfiinc  , EHrphic  ,
  		       EHtrkid, EHevt    , EHzc    , EHmevt   , EHrphixp1,
  		       EHzxp1 , EHrphixp2, EHzxp2  , EHismulti, EHissplit};

  
  using PadInfo_t = std::tuple<int   ,  // module
  		               int   ,  // row
  	     		       double,  // normchg
  			       double,  // drphi
  			       double,  // dt
  			       double,  // chgsum
  		               int   ,  // ndf
  		               double,  // chi2
  		               double,  // fi0
  		               double,  // cpa
  		               double,  // tnl
  			       int   ,  // ntrks
  		               double,  // fi0loc
  		               double,  // fiinc
  			       int   ,  // trkid
  			       int   ,  // evt
			       double,  // rphic
			       double,  // zc
  			       int   ,  // mevt
  		       	       double,  // rphixp1
  		       	       double,  // zxp1
  		       	       double,  // rphixp2
  		       	       double>; // zxp2

  using TrackInfoMap_t = std::map<int,		// track # 
  	     			  TrackInfo_t   // track info
  			         >;
  
  using HitInfoMap_t = std::map<int,		   // track # 
  	 		        std::map<int,      // serial row #
  		  		         HitInfo_t // hit info
                   		        >
  			       >;
  
  using PadInfoMap_t = std::map<int,		   	        // track # 
  	 	                std::map<int,          	        // serial row #
  		                         std::vector<PadInfo_t> // pad info vector
                   	                >
  			       >;
 
  /// LCIO's virtual functions to override.

  virtual Processor*  newProcessor() { return new RootFileProcessor2 ; }
  
  
  RootFileProcessor2() ;
  
  /** Called at the begin of the job before anything is read.
   * Use to initialize the processor, e.g. book histograms.
   */
  virtual void init() ;
  
  /** Called for every run.
   */
  virtual void processRunHeader( LCRunHeader* run ) ;
  
  /** Called for every event - the working horse.
   */
  virtual void processEvent( LCEvent * evt ) ; 
  
  
  virtual void check( LCEvent * evt ) ; 
  
  
  /** Called after data processing for clean up.
   */
  virtual void end() ;
  
 private:
  /// Utility functions.
  int  getModuleID   (const EVENT::TrackerHit &hit);
  int  getRowID      (const EVENT::TrackerHit &hit, const gear::TPCModule & tpcmodule);
  int  getSerialRowNo(int modID, int rowID);
  bool isMultiHit(const EXTPCHit &kalhit);
  bool isSplitHit(const EXTPCHit &kalhit);

  /// Initialize event information map
  void initInfoMaps(HitInfoMap_t   & hitInfoMap1,
		    HitInfoMap_t   & hitInfoMap2,
		    TrackInfoMap_t & trackInfoMap,
		    HitInfoMap_t   & hitInfoMap,
		    PadInfoMap_t   & padInfoMap);

  /// Get event information map
  void getInfoMaps(LCCollection   * colTracksPtr,
		   TrackInfoMap_t & trackInfoMap,
		   HitInfoMap_t   & hitInfoMap,
		   PadInfoMap_t   & padInfoMap,
		   bool             ismerged = false);

  /// fill track info ntuple
  void fillTupTrk(const int            i_trk,      // track #
		        AIDA::ITuple & trks,       // tuple to fill
			TrackInfo_t  & trackInfo); // track info

  /// fill hit info ntuple
  void fillTupHit(const int            i_trk,    // track #
		  const int            irow,     // serial row #
			AIDA::ITuple & hits,     // tuple to fill
			HitInfo_t    & hitInfo); // hit info

  void fillTupClr(const int                      i_trk,       // track #
  			AIDA::ITuple           & clrs,        // tuple to fill
  			std::vector<PadInfo_t> & padInfoVec); // pad info vector

  /// Fill output n-tuples
  void fillTuples(const int              nTrks,
		        AIDA::ITuple   & trks,
		        AIDA::ITuple   & hits,
		        AIDA::ITuple   & clrs,
		        TrackInfoMap_t & trackInfoMap,
		        HitInfoMap_t   & hitInfoMap,
		        PadInfoMap_t   & padInfoMap);

  
  /// Create KalHit from TrackerHit.
  EXTPCHit   *createKalHit(const EVENT::TrackerHit *hitp, bool ismerged = false);
  
  /// Create KalTrack from TrackerHitVec by Kalman Filtering it.
  EXHYBTrack *createKalTrack(TObjArray          &kalhits,
		       const bool                dir,
		             std::map<int, int> &nhits);
  
  /// Calculate track's incident angle and expected hit position corresponding to the hit in question.
  std::tuple<double, TVector3>
  	getIncidentPhiAndPosition(EXHYBTrack      &kaltrk,     // KalTrack
  	                    const EXTPCHit        &hit,        // KalHit
  	                    const gear::TPCModule &tpcmodule);

  /// Calculate expected hit position at a given row
  TVector3 getExpectedHitPosition(const EVENT::Track &trk, int serialrow);
    
 protected:

  /** Input collection name.
   */
  std::string _inputColName;
  std::string _curTrackColName;
  std::string _mergedRawDataColName;

  /** EXTPCKalDetector
   */

#ifdef MARLIN_USE_AIDA
  /** Used to prepare Ntuple
   */
  void setNtuple();

  /** Ntuple for analysis 
   */
  AIDA::ITuple *_clrs ; // clusters
  AIDA::ITuple *_hits ; // hits
  AIDA::ITuple *_trks ; // tracks

  AIDA::ITuple *_clrs2; // clusters
  AIDA::ITuple *_hits2; // hits
  AIDA::ITuple *_trks2; // tracks

  std::map<std::string,					  // key1: unmerged/merged
	   std::map<std::string,			  // key2: trks/hits/clrs 
	  	    std::tuple<std::string, std::string>  // (path, title)
		   >
	  > _tuplePathTitleVec; 		    // tuple path and title holder
#endif

  int _nRun ;
  int _nEvt ;
  int _nMergedEvt ;
  int _nKalTrks ;

  std::vector<int> _modulesToUseOverride;
  std::vector<int> _badRowsOverride;

  std::vector<const gear::TPCModule *> _moduleVec; // module geometry array

  double _maxDriftLength;
};

#endif
