#include "RefinedTrack.h"
#include "IMPL/TrackImpl.h"
#include "IMPL/TrackerHitImpl.h"
#include "IMPL/TrackStateImpl.h"

#include "kaltest/TKalTrack.h"
#include "kaltest/TKalTrackSite.h"
#include "kaltest/TKalTrackState.h"

#include <map>

namespace marlintpc {
	/// C-tors
	RefinedTrack::RefinedTrack()
	{
	}

	RefinedTrack::RefinedTrack(const IMPL::TrackImpl 				             *trkp,
	                           const std::map<const EVENT::TrackerHit *, IMPL::TrackerHitImpl *> &theInputHitToOutputHitMap)
	{
		setType                (trkp->getType                ());
		setChi2                (trkp->getChi2                ());
		setNdf                 (trkp->getNdf                 ());
		setdEdx                (trkp->getdEdx                ());
		setdEdxError           (trkp->getdEdxError           ());
		setRadiusOfInnermostHit(trkp->getRadiusOfInnermostHit());

		auto &subdNos = trkp->getSubdetectorHitNumbers();
    		std::copy(subdNos.begin(), subdNos.end(), std::back_inserter(_subdetectorHitNumbers));

		auto &tracks  = trkp->getTracks();
    		std::copy(tracks.begin(), tracks.end(), std::back_inserter(_tracks));

		for (auto oldhitp : trkp->getTrackerHits()) {
			auto hitp = theInputHitToOutputHitMap.at(oldhitp);
			_hits.push_back(hitp);
		}

		auto states = trkp->getTrackStates();

    		_trackStates.reserve(states.size());
    		for (unsigned int i = 0; i < states.size(); i++) {
    			_trackStates.push_back(new IMPL::TrackStateImpl(*trkp->getTrackState(i)));
    		}
	}

	void RefinedTrack::modify(TKalTrack *kaltrkp)
	{
    		auto &cursite  = static_cast<TKalTrackSite &>(kaltrkp->GetCurSite());
    		auto &trkState = dynamic_cast<TKalTrackState &>(kaltrkp->GetCurSite().GetCurState());
		auto  helix    = trkState.GetHelix();
		auto &covMat   = trkState.GetCovMat();

		auto chi2      = kaltrkp->GetChi2();
		auto ndf       = kaltrkp->GetNDF();

		auto toBaseRange = [](double phi) 				// lambda function
		{
			while (phi <= -M_PI)
			{
				phi += 2. * M_PI ;
			}
			while (phi >   M_PI)
			{
				phi -= 2. * M_PI ;
			}
			return phi ;
		};
    		auto phi       =   toBaseRange(helix.GetPhi0() + M_PI / 2.) ;
		auto kappa     =   helix.GetKappa();
    		auto omega     =   1. / helix.GetRho()  ;
    		auto d0        = - helix.GetDrho() ;
    		auto z0        =   helix.GetDz()   ;
    		auto tanLambda =   helix.GetTanLambda()  ;

    		auto alpha = kappa / omega;

    		EVENT::FloatVec cov(15);
    		cov[ 0] =   covMat(0, 0);   		  //   d0,   d0

    		cov[ 1] = - covMat(1, 0);   		  //   phi0, d0
    		cov[ 2] =   covMat(1, 1);   		  //   phi0, phi

    		cov[ 3] = - covMat(2, 0) * alpha;         //   omega, d0
    		cov[ 4] =   covMat(2, 1) * alpha;         //   omega, phi
    		cov[ 5] =   covMat(2, 2) * alpha * alpha; //   omega, omega

    		cov[ 6] = - covMat(3, 0);   		  //   z0  , d0
    		cov[ 7] =   covMat(3, 1);   		  //   z0  , phi
    		cov[ 8] =   covMat(3, 2) * alpha;   	  //   z0  , omega
    		cov[ 9] =   covMat(3, 3);  		  //   z0  , z0

    		cov[10] = - covMat(4, 0);   		  //   tanl, d0
    		cov[11] =   covMat(4, 1);   		  //   tanl, phi
    		cov[12] =   covMat(4, 2) * alpha;   	  //   tanl, omega
    		cov[13] =   covMat(4, 3);   		  //   tanl, z0
    		cov[14] =   covMat(4, 4);   		  //   tanl, tanl

    		setChi2(chi2) ;
    		setNdf(ndf) ;

    		setD0(d0) ;
    		setPhi(phi) ;    // fi0  - M_PI/2.  ) ;
    		setOmega(omega) ;
    		setZ0(z0) ;
    		setTanLambda(tanLambda) ;
    		setCovMatrix(cov) ;

    		float pivot[] = { float(cursite.GetPivot()(0)),
    				  float(cursite.GetPivot()(1)),
    				  float(cursite.GetPivot()(2)) };

    		setReferencePoint(pivot) ;
	}
}
